#!/bin/sh
# Update Slackware 8.0 installation to Slackware 8.1
# Copyright (C) 2002 Georgi Chorbadzhiyski (georgi AT top.bg)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

. ./config.sh

clearpath=' sed -e s|.*/||;s|.tgz||'

function upgrade_pkg_on() {
	old=`echo $1 | $clearpath`
	new=`echo $2 | $clearpath`
	newdir=`echo $2 | sed -e 's|/.*||'`
	echo "( cd $SLACKDIR/$newdir && upgradepkg $old%$new )"
}

function upgrade_pkg() {
	new=`echo $1 | $clearpath`
	newdir=`echo $1 | sed -e 's|/.*||'`
	cd $SLACKDIR/$newdir
	echo "( cd $SLACKDIR/$newdir && upgradepkg $new )"
}

function install_pkg() {
	new=`echo $1 | $clearpath`
	newdir=`echo $1 | sed -e 's|/.*||'`
	cd $SLACKDIR/$newdir
	echo "( cd $SLACKDIR/$newdir && installpkg $new )"
}

function remove_pkg() {
	old=`echo $1 $clearpath`
	echo "removepkg $old"
}

# Check if package is installed (can check for short
# package names and for long package names)
function is_installed() {
	pkg_name=`echo $1 | $clearpath`
	pkg_name_base=`echo $pkg_name | rev | cut -d- -f4- | rev`
	if [ "$pkg_name_base" != "" ]; then
		pkg_name_base=$pkg_name
	fi
	pkg_name_full=`ls ${PKGDIR}/${pkg_name_base}-[0-9]* 2>/dev/null | $clearpath`
	if [ "$pkg_name_full" = "" ]; then
		pkg_name_full=$pkg_name
	fi
	if [ ! -e "${PKGDIR}/${pkg_name}" ]; then
		if [ ! -e "${PKGDIR}/${pkg_name_full}" ]; then
			if [ ! -e "${PKGDIR}/${pkg_name_base}" ]; then
				return 1
			fi
		fi
	fi
	return 0
}

# Check if package is installed
# Looks for exact match
function is_pkg_installed() {
	pkg_name=`echo $1 | $clearpath`
	if [ ! -e "${PKGDIR}/${pkg_name}" ]; then
		return 1
	fi
	return 0
}

# If package is not installed install it
function add_pkg() {
	pkg=$1
	is_pkg_installed $pkg
	if [ $? = 1 ]; then # If not alrady installed
		install_pkg $pkg
	else
		echo "# *** $pkg already installed ***"
	fi
}

# Install package if it is not already installed
# and dependent package is installed
function addif_pkg() {
	pkg=$1
	ifpkg=$2
	is_installed $ifpkg
	if [ $? = 0 ]; then # If dependant package is installed
		add_pkg $pkg
	else
		echo "# *** $pkg not added because $ifpkg is not installed ***"
	fi
}

# Upgrade package if not already upgrated to current version
function up_pkg() {
	oldpkg=$1
	newpkg=$2
	is_installed $oldpkg
	if [ $? = 0 ]; then # Not installed - proceed with updates
		is_pkg_installed $newpkg
		if [ $? = 1 ]; then # Incomming package is not installed
			is_pkg_installed $oldpkg
			if [ $? = 0 ]; then # Upgrade to new package
				upgrade_pkg_on ${oldpkg} ${newpkg}
			else # Old package is not really that old ;)
				upgrade_pkg ${newpkg}
			fi
		fi
	fi
}

# Remove installed package
function rm_pkg() {
	pkg=$1
	is_installed $pkg
	if [ $? = 0 ]; then
		remove_pkg $pkg
	fi
}

# The main loop
cat $INFILE | grep -v ^# | grep -v ^$ | \
while read op pkg1 pkg2; do
	case "$op" in
		U)
			up_pkg $pkg1 $pkg2
		;;
		A)
			add_pkg $pkg1
		;;
		AI)
			addif_pkg $pkg1 $pkg2
		;;
		R)
			rm_pkg $pkg1
		;;
		*)
			echo "# *** Unknown operation: $op ***"
	esac
done
